@@ -1,9 +1,10 @@
-import pandas as pd
 import numpy as np
-from .utils import input_series
+import pandas as pd
 from scipy.sparse import issparse
 from tqdm import tqdm
 
+from .utils import input_series
+
 
 class Model:
     def __init__(self, name="Model", verbose=True):
@@ -12,6 +13,14 @@ class Model:
         self.layers = []
         self.layer2num = {}
 
+    def unload(self):
+        for layer in self.layers:
+            layer.unload()
+
+    def reload(self, **kwargs):
+        for layer in self.layers:
+            layer.reload(**kwargs)
+
     def add(self, layer):
         layer.verbose = self.verbose
         layer_name = layer.name
@@ -69,15 +78,7 @@ class Model:
 
     def save(self, filename):
         import dill
-
-        # check if model has keras layer
-        for layer in self.layers:
-            if layer.__class__.__name__ != "Keras":
-                continue
-            layer.weights = layer.model.get_weights()
-            layer.config = layer.model.get_config()
-            del layer.model
-
+        self.unload()
         with open(filename, "wb") as f:
             dill.dump(self, f)
 
@@ -102,6 +103,12 @@ class Layer:
         self.trained = False
         self.run_parallel = parallel
 
+    def unload(self):
+        pass
+
+    def reload(self, **_):
+        pass
+
     def process(self, df, y=None):
         if self.input is None:
             raise KeyError(f"{self.name}: no input defined")
@@ -153,6 +160,7 @@ class Layer:
 
     def save(self, filename):
         import dill
+        self.unload()
         with open(filename, "wb") as f:
             dill.dump(self, f)
 
@@ -160,9 +168,6 @@ class Layer:
 class WordVectors:
     def __init__(
         self,
-        word2id=None,
-        id2word=None,
-        weights=None,
         model=None
     ):
         if model is not None:
@@ -217,41 +222,11 @@ class WordVectors:
 # Functions
 # =============================================================================
 
-def load_model(filename, custom_objects=None):
+def load_model(filename, **kwargs):
     import dill
     with open(filename, "rb") as f:
         obj = dill.load(f)
-
-    if isinstance(obj, Model):
-        # check if model has keras layer
-        for layer in obj.layers:
-            if layer.__class__.__name__ != "Keras":
-                continue
-            from tensorflow.keras.models import Sequential
-            from tensorflow.keras.models import Model as KModel
-            try:
-                model = KModel.from_config(
-                    layer.config, custom_objects=custom_objects)
-            except KeyError:
-                model = Sequential.from_config(
-                    layer.config, custom_objects=custom_objects)
-            model.set_weights(layer.weights)
-            del layer.weights
-            del layer.config
-            layer.model = model
-    elif obj.__class__.__name__ == "Keras":
-        from tensorflow.keras.models import Sequential
-        from tensorflow.keras.models import Model as KModel
-        try:
-            model = KModel.from_config(
-                obj.config, custom_objects=custom_objects)
-        except KeyError:
-            model = Sequential.from_config(
-                obj.config, custom_objects=custom_objects)
-        model.set_weights(obj.weights)
-        del obj.weights
-        del obj.config
-        obj.model = model
+    obj.reload(**kwargs)
     return obj
 
 
