@@ -1,12 +1,13 @@
-from .. import Layer
-from functools import partial
 import re
+from functools import partial
 
+from .. import Layer
 
 # =============================================================================
 # Layers
 # =============================================================================
 
+
 class Tokenize(Layer):
     parallel = True
     trainable = False
@@ -62,11 +63,19 @@ class Snowball(Layer):
         parallel=False
     ):
         super(Snowball, self).__init__(input, output, name, verbose, parallel)
-        from nltk.stem.snowball import SnowballStemmer
-        self.stemmer = SnowballStemmer(lang)
+        self.lang = lang
+        self.reload()
         self.memoize = memoize
         self.word2stem = {}
 
+    def unload(self):
+        del self.stemmer
+        self.word2stem = {}
+
+    def reload(self, **_):
+        from nltk.stem.snowball import SnowballStemmer
+        self.stemmer = SnowballStemmer(self.lang)
+
     def process_doc(self, text):
         if self.memoize:
             words = []
@@ -101,11 +110,11 @@ class Phrase(Layer):
         self.min_count = min_count
         self.threshold = threshold
 
-    def fit(self, series, y=None):
+    def fit(self, series, _=None):
         self.pmi = set(pmi(
             series, window_size=1,
             minimum=self.threshold,
-            threshold=self.min_count).keys())
+            min_count=self.min_count).keys())
 
     def process_doc(self, text):
         if len(text) == 0:
@@ -150,10 +159,30 @@ class Sub(Layer):
         return re.sub(self.regex, self.replacement, doc)
 
 
+class Contract(Layer):
+    parallel = True
+    trainable = False
+
+    def __init__(
+        self,
+        input=None,
+        output=None,
+        name=None,
+        verbose=True,
+        parallel=False
+    ):
+        super(Contract, self).__init__(input, output, name, verbose, parallel)
+
+    def process_doc(self, doc):
+        if not isinstance(doc, str):
+            doc = str(doc)
+        return re.sub(r"(.)\1{2,}", r"\1", doc)
+
 # =============================================================================
 # Functions
 # =============================================================================
 
+
 def tokenize(
     text,
     stopwords=None,
@@ -188,10 +217,10 @@ def tokenize(
     return words
 
 
-def pmi(series, window_size=3, threshold=2, minimum=0.6):
-    from collections import Counter, defaultdict
+def pmi(series, window_size=3, min_count=2, minimum=0.6, normalize=False):
     import itertools
     import math
+    from collections import Counter, defaultdict
 
     # compute freq
     freq = Counter(itertools.chain(*series))
@@ -214,14 +243,17 @@ def pmi(series, window_size=3, threshold=2, minimum=0.6):
 
     npmi_ = {}
     for couple, count in cooc.items():
-        if count < threshold:
+        if count < min_count:
             continue
         x, y = couple
         p_x = freq[x]
         p_y = freq[y]
         p_xy = count / M
         prod = p_x * p_y / (N*N)
-        npmi_value = math.log(p_xy / prod)
+        if normalize:
+            npmi_value = math.log(prod) / math.log(p_xy) - 1
+        else:
+            npmi_value = math.log(p_xy / prod)
         if npmi_value > minimum:
             npmi_[couple] = npmi_value
     return npmi_
